from typing import Callable, Literal

def build_message(role: Literal['system', 'user', 'assitant'], content): ...
def ai_template(func: Callable | None = None, post_process: Callable | None = None): ...
